
package org.act.services;
import org.act.metaset.Attribute;
import org.act.metaset.Metaset;
import org.act.metaset.QName;
import org.act.services.pool.IObjectPool;
import org.act.services.protocol.http.
       HTTPReplyTransportContext;
import org.act.services.runtime.IRuntime;
import org.act.services.soap.model.Body;
import org.act.services.soap.model.Envelope;
import org.act.services.soap.model.Header;
import org.act.services.soap.model.SoapAttribute;
import org.act.services.soap.model.SoapQName;

public class MessageContext extends AMessageContext {
	
	
	public MessageContext() {
	}
	
	
	public MessageContext(String op, String url, 
						  IRuntime runtime) {
	generateSOAP(op, url, runtime);
	}
	
	
	public MessageContext(String operationName, 
			Metaset params, Metaset correlationSets,
			String url, IRuntime runtime,
			Metaset tranContextServicesSet) {
	generateSOAP(operationName, params, correlationSets,
			url, runtime, tranContextServicesSet);
	}
	
	
	private IRuntime runtime = null;
	
	
	public String getMessageProtocolName() {
	return "SOAP";
	}
	
	
	private boolean generateSOAP() {
	boolean result = false;
	try {
				Envelope env = new Envelope("soapenv", "Envelope");
		env.add(new SoapAttribute(
				new SoapQName("xmlns", "soapenv"),
				"http://schemas.xmlsoap.org/soap/envelope/"));
		env.add(new SoapAttribute(
				new SoapQName("xmlns", "xsi"),
				"http://www.w3.org/2001/XMLSchema-instance"));
		env.add(new SoapAttribute(
				new SoapQName("xmlns", "xsd"),
				"http://www.w3.org/2001/XMLSchema"));
		env.add(new SoapAttribute(
				new SoapQName("xmlns", "soapenc"),
				"http://schemas.xmlsoap.org/soap/encoding/"));
		Header head = new Header("soapenv", "Header");
		env.add(head);
		Body body = new Body("soapenv", "Body");
		env.add(body);
				super.setResponseServices(env);
		result = true;;
	} catch (Exception e) {
		e.printStackTrace();
		result = false;
	}
	return result;
	}
	
	
	private boolean addSoapHeaderElement(Metaset hElement) {
	boolean result = false;
	try {
		if (hElement != null) {
						Header head = (Header)(super.getResponseServices().
					filter(".\\Header") ).getChilds().get(0);
			head.add(hElement);
		}
		result = true;
	} catch (Exception e) {
		e.printStackTrace();
		result = false;
	}
	return result;
	}
	
	
	private boolean addSoapBodyElement(Metaset bElement) {
	boolean result = false;
	try {
		if (bElement != null) {
						Body body = (Body)(super.getResponseServices().
					filter(".\\Body") ).getChilds().get(0);
			body.add(bElement);
		}
		result = true;
	} catch (Exception e) {
		e.printStackTrace();
		result = false;
	}
	return result;
	}
	
	
	private boolean putToMessageOUTpool() {
	boolean result = false;
	try {
				IObjectPool msgOutputPool = 
			this.runtime.getObjectPool
			("message",	IObjectPool.OUT_MODE);
		synchronized (msgOutputPool) {
			msgOutputPool.put(this);
			msgOutputPool.notifyAll();
		}
		result = true;
	} catch (Exception e) {
		e.printStackTrace();
		result = false;
	}
	return result;
	}
	
	
	public Object clone() throws 
						  CloneNotSupportedException {
	MessageContext mc = (MessageContext)super.clone();
				return mc;
	}
	
	public boolean generateSOAP(Header head, Body body,
								IRuntime runtime) {
	boolean result = false;
	this.runtime = runtime;
	try {
				if ( generateSOAP() ) {
			if ( addSoapHeaderElement(head) ) {
				if ( addSoapBodyElement(body) ) {
					if ( putToMessageOUTpool() ) {
						result = true;
					}
				}
			}
		}
	} catch (Exception e) {
		e.printStackTrace();
		result = false;
	}
	return result;
	}
	
	
	public boolean generateSOAP(Header head, Body body, 
			                    String url, IRuntime runtime) {
	boolean result = false;
	this.runtime = runtime;
	try {
				if ( generateSOAP() ) {
			if ( addSoapHeaderElement(head) ) {
				if ( addSoapBodyElement(body) ) {
					IObjectPool pool = this.runtime.
					getObjectPool("transport", 
					IObjectPool.IN_MODE);
					this.setResponseTransportContext(
							new HTTPReplyTransportContext(
									url, pool));
					if ( putToMessageOUTpool() ) {
						result = true;
					}
				}
			}
		}
	} catch (Exception e) {
		e.printStackTrace();
		result = false;
	}
	return result;
	}
	
	public boolean generateSOAP(MessageContext mc,
			String operationName, 
			Metaset params, Metaset correlationSets,
			IRuntime runtime
			) {
	boolean result = true;
	this.runtime = runtime;
	try {
		Envelope env = new Envelope("soapenv", "Envelope");
		env.add(new SoapAttribute(
				new SoapQName("xmlns", "soapenv"),
				"http://schemas.xmlsoap.org/soap/envelope/"));
		env.add(new SoapAttribute(
				new SoapQName("xmlns", "xsi"),
				"http://www.w3.org/2001/XMLSchema-instance"));
		env.add(new SoapAttribute(
				new SoapQName("xmlns", "xsd"),
				"http://www.w3.org/2001/XMLSchema"));
		env.add(new SoapAttribute(
				new SoapQName("xmlns", "soapenc"),
				"http://schemas.xmlsoap.org/soap/encoding/"));
		Header head = new Header("soapenv", "Header");
		
		if (correlationSets != null)
			head.add(correlationSets);
		env.add(head);
		
		Body body = new Body("soapenv", "Body");
		Metaset res = new Metaset(operationName + "Response");
		for (int i = 0; i < params.getChilds().size(); i++) {
			Metaset item = (Metaset) params.getChilds().get(i);
			String value = item.getValue();
			Attribute type = (Attribute) item.getAttributes().get(1);
			String typeValue = type.getValue();
			Metaset part = new Metaset(operationName + "Return");
			part.setValue(value);
			part.add(new Attribute("xsi:type", typeValue));
			res.add(part);
		}
		body.add(res);
		env.add(body);
	    
		mc.setResponseServices(env);
		try {
						IObjectPool msgOutputPool = 
				this.runtime.getObjectPool
				("message",	IObjectPool.OUT_MODE);
			synchronized (msgOutputPool) {
				msgOutputPool.put(mc);
				msgOutputPool.notifyAll();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	} catch (Exception ex) {
		ex.printStackTrace();
		result = false;
	}
	return result;
	}
	
	
	public boolean generateSOAP(String operationName, 
			Metaset params, Metaset correlationSets,
			String url, IRuntime runtime,
			Metaset tranContextServicesSet) {
	boolean result = false;
	this.runtime = runtime;
	try {
				generateSOAP();
		addSoapHeaderElement(correlationSets);
		addSoapHeaderElement(tranContextServicesSet);
		Metaset res = new Metaset(operationName);
		for (int i=0; i<params.getChilds().size(); i++) {
			Metaset item = (Metaset)params.getChilds().get(i);
						String value = item.getValue();
						String typeValue = item.selector(".\\", "type");
						Metaset part = new Metaset(
					item.selector(".\\", "name"));
						part.setValue(value);
						part.add(new Attribute("xsi:type", typeValue));
					    res.add(part);
		}
		addSoapBodyElement(res);
		IObjectPool pool = this.runtime.
		getObjectPool("transport", 
		IObjectPool.IN_MODE);
		this.setResponseTransportContext(
				new HTTPReplyTransportContext(url, pool));
		putToMessageOUTpool();
		result = true;
	} catch (Exception e) {
		e.printStackTrace();
		result = false;
	}
	return result;
	}
	
	public boolean generateSOAP(String operationName, 
								String url, 
								IRuntime runtime) {
	boolean result = false;
	this.runtime = runtime;
	try {
				generateSOAP();
		Metaset operation = new Metaset(operationName);
		Metaset parameter = new Metaset("param0");
		parameter.add(new Attribute(new QName("xsi", "type"),
						"xsd:string"));
		parameter.setValue("zhuyan");
		operation.add(parameter);
		addSoapBodyElement(operation);
		IObjectPool pool = this.runtime.
						   getObjectPool("transport", 
		                   IObjectPool.IN_MODE);
		this.setResponseTransportContext(
				new HTTPReplyTransportContext(
						url, pool));
		putToMessageOUTpool();
		result = true;
	} catch (Exception e) {
		e.printStackTrace();
		result = false;
	}
	return result;
	}
}
