
package org.act.services.connector;
import java.util.ArrayList;
import java.util.List;
import org.act.metaset.Metaset;
import org.act.services.ConfigException;
import org.act.services.ServicesObject;
import org.act.services.pool.IObjectPool;
import org.act.services.protocol.ProviderProtocol;
import org.act.services.runtime.IRuntime;
import org.act.services.thread.ObjectOperationException;
import org.act.services.thread.ObjectProducerThreadPoolRunnable;
import org.act.services.thread.Terminatable;
import org.act.services.thread.ThreadPool;

public class ProviderConnector extends AObjectPoolConnector {
	public ProviderConnector(IRuntime runtime) {
		super(runtime);
	}
	private IObjectPool soIn;
	private IObjectPool soOut;
	private Terminatable provider;
	private ThreadPool threadPool;
	private List providers = new ArrayList();
	
	public void config(Metaset conf) throws ConfigException {
		super.config(conf);
		String in = conf.selector(".\\","inputpool");
		String out = conf.selector(".\\","outputpool");
		soIn = runtime.getObjectPool(in, IObjectPool.IN_MODE);
		soOut = runtime.getObjectPool(out, IObjectPool.OUT_MODE);
		
				String tpName = conf.selector(".\\threadpool","name");		threadPool = runtime.getThreadPool(tpName);		
				Metaset protocols = conf.filter(".\\protocol");
		for (int i=0; i<protocols.getChilds().size(); i++) {
			try {
				Metaset protocol = (Metaset) protocols.getChilds().get(i);
				String classname = protocol.selector(".\\","class");
				Class cls = Class.forName(classname);
				if (ProviderProtocol.class.isAssignableFrom(cls)) {
					ProviderProtocol providerProtocol = (ProviderProtocol) cls.newInstance();
					providerProtocol.config(protocol);
					providers.add(providerProtocol);
				}
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (ConfigException e) {
				e.printStackTrace();
			}
		}
		
		provider = new Terminatable() {
			protected void process() throws ObjectOperationException {
								ServicesObject so = null;
				synchronized (soIn) {
					while (soIn.empty()) {
						switch (getState()) {
							case RUNNING :
								try {
									soIn.wait();
								} catch (InterruptedException e) {
									e.printStackTrace();
								}
								break;
							case STOPPING :
								return;
							default :
								throw new IllegalMonitorStateException();
						}
					}
					so = (ServicesObject) soIn.get();
					soIn.notifyAll();
				}
				
								for (int i=0; i<providers.size(); i++) {
					ProviderProtocol comsumer = (ProviderProtocol) providers.get(i);
					Object adaptee = comsumer.adapt(so);
					if (adaptee!=null && adaptee instanceof ObjectProducerThreadPoolRunnable) {
						ObjectProducerThreadPoolRunnable runnable = (ObjectProducerThreadPoolRunnable)adaptee;
						runnable.setReceiver(soOut);
						threadPool.runIt(runnable);
						break;
					}
				}
			}		
		};
	}
	public void stop() {
		provider.terminate();
		synchronized (soIn) {
			soIn.notifyAll();
		}
	}
	public void start() {
		Thread t  = new Thread(provider);		t.setName("Provider");
		t.start();
	}
	public Object adapt(Object obj) {
		return null;
	}
}
