
package org.act.services.transaction.messages;
import org.act.services.transaction.util.*;
import java.util.*;


public class ConfirmTransaction extends Message{
	 
    private String transactid_;
    
    private Vector inflist_ = null;
    
    private boolean reporthazard_ = true;
    
    public ConfirmTransaction(String trid)
    {
        super("btp:confirm-transaction", Message.TYPE_CONFIRM_TRANSACTION);
        transactid_ = trid;
    }
    
    public ConfirmTransaction(String trid, boolean reporthazard)
    {
        super("btp:confirm-transaction", Message.TYPE_CONFIRM_TRANSACTION);
        transactid_ = trid;
        reporthazard_ = reporthazard;
    }
    
    public ConfirmTransaction(XElement el)
    {
        super(el, Message.TYPE_CONFIRM_TRANSACTION);
        String ns = el.getNamespace();
        ns = (ns == null ? "" : ns + ":");
        XHashtable h = el.toHashtable();
        id_ = el.getAttribute("id");
        transactid_ = pickString(h,
                                 "/" + name_ + "/" + ns +
                                 "transaction-identifier");
        Object obj = h.getSingle("/" + name_ + "/" + ns + "report-hazard");
        if (obj != null)
        {
            reporthazard_ = !obj.toString().equalsIgnoreCase("false");
        }
        inflist_ = h.get("/" + name_
                         + "/" + ns + "inferiors-list" + "/" + ns +
                         "inferior-identifier");
    }
    
    public String getTransactionIdentifier()
    {
        return transactid_;
    }
    
    public boolean getReportHazard()
    {
        return reporthazard_;
    }
    
    public void addInferiorIdentifier(String id)
    {
        if (inflist_ == null)
        {
            inflist_ = new Vector();
        }
        inflist_.addElement(id);
    }
    
    public void setInferiorIdentifiers(Vector inferiorList)
    {
        this.inflist_ = inferiorList;
    }
    
    public Vector getInferiorIdentifiers()
    {
        return inflist_;
    }
    
    public String getInferiorIdentifier(int pos)
    {
        if (inflist_ == null || pos >= inflist_.size())
        {
            return null;
        }
        return (String)inflist_.elementAt(pos);
    }
    
    public XElement toXml()
    {
        XElement el = new XElement("btp:confirm-transaction");
        if (id_ != null)
        {
            el.addAttribute("id", id_);
        }
        el.addElement(
            new XElement("btp:transaction-identifier", transactid_));
        el.addElement(
            new XElement("btp:report-hazard", (reporthazard_ ? "true" : "false")));
        if (inflist_ != null)
        {
            XElement a = new XElement("btp:inferiors-list");
            for (int i = 0; i < inflist_.size(); i++)
            {
                a.addElement(
                    new XElement("btp:inferior-identifier",
                                 (String)inflist_.elementAt(i)));
            }
            el.addElement(a);
        }
        addQualifiers(el);
        addTargetAddress(el);
        addReplyAddress(el);
        return el;
    }
    public static void main(String args[])
    {
        ConfirmTransaction obj = new ConfirmTransaction("transact1", true);
        obj.setTransactionTimelimit(1000);
                obj.setInferiorName("name");
                System.out.println(obj.toXml().toString());
    }
}
