
package org.act.services.transaction.messages;
import org.act.services.transaction.util.*;
import java.util.*;


public class Enrol extends Message{
	
    private boolean responseRequested_ = false;
    
    private String superiorIdentifier_ = null;
    
    private String inferiorIdentifier_ = null;
    
    Vector inferioraddr_ = null;
    
    private String compensationAddress = null;
    
    public Enrol()
    {
        super("btp:enrol", Message.TYPE_ENROL);
    }
    
    public Enrol(XElement el)
    {
        super(el, Message.TYPE_ENROL);
        String ns = el.getNamespace();
        ns = (ns == null ? "" : ns + ":");
        XHashtable h = el.toHashtable();
        id_ = el.getAttribute("id");
        Object obj = h.getSingle("/" + name_ + "/" + ns + "response-requested");
        if (obj != null)
        {
            responseRequested_ = !obj.toString().equalsIgnoreCase("false");
        }
        superiorIdentifier_ = pickString(h,
                                         "/" + name_ + "/" + ns +
                                         "superior-identifier");
        inferiorIdentifier_ = pickString(h,
                                         "/" + name_ + "/" + ns +
                                         "inferior-identifier");
        inferioraddr_ = pickAddrSet(h,
                                    "/" + name_ + "/" + ns + "inferior-address");
    }
    
    public void setResponseRequested(boolean response)
    {
        responseRequested_ = response;
    }
    
    public boolean getResponseRequested()
    {
        return responseRequested_;
    }
    
    public void setSuperiorIdentifier(String uri)
    {
        superiorIdentifier_ = uri;
    }
    
    public String getSuperiorIdentifier()
    {
        return superiorIdentifier_;
    }
    
    public void setInferiorIdentifier(String uri)
    {
        inferiorIdentifier_ = uri;
    }
    
    public String getInferiorIdentifier()
    {
        return inferiorIdentifier_;
    }
    
    public XElement toXml()
    {
        XElement el = new XElement("btp:enrol");
        if (id_ != null)
        {
            el.addAttribute("id", id_);
        }
        el.addElement(
            new XElement("btp:superior-identifier", superiorIdentifier_));
        el.addElement(
            new XElement("btp:response-requested",
                         (responseRequested_ ? "true" : "false")));
        addInferiorAddress(el);
        el.addElement(
            new XElement("btp:inferior-identifier", inferiorIdentifier_));
        addQualifiers(el);
        addTargetAddress(el);
        addReplyAddress(el);
        return el;
    }
    
    public void addInferiorAddress(Address add)
    {
        if (inferioraddr_ == null)
        {
            inferioraddr_ = new Vector();
        }
        inferioraddr_.addElement(add);
    }
    
    public Vector getInferiorAddresses()
    {
        return inferioraddr_;
    }
    
    public int getInferiorAddressesCount()
    {
        return (inferioraddr_ == null ? 0 : inferioraddr_.size());
    }
    
    public Address getInferiorAddress(int pos)
    {
        if (inferioraddr_ == null || pos >= inferioraddr_.size())
        {
            return null;
        }
        else
        {
            return (Address)inferioraddr_.elementAt(pos);
        }
    }
    
    protected void addInferiorAddress(XElement el)
    {
        if (inferioraddr_ != null)
        {
            for (int i = 0; i < inferioraddr_.size(); i++)
            {
                XElement inf = new XElement("btp:inferior-address");
                inf.addElement(((Address)inferioraddr_.elementAt(i)).toXml());
                el.addElement(inf);
            }
        }
    }
    
    public void setCompensationAddress(String addr)
    {
        this.compensationAddress = addr;
    }
    
    public String getCompensationAddress()
    {
        return this.compensationAddress;
    }
    public static void main(String args[])
    {
        Enrol obj = new Enrol();
        obj.setSuperiorIdentifier("http://www.myaddress.com:8080/superiorid");
        obj.setResponseRequested(false);
        obj.setInferiorIdentifier("http://www.myaddress.com:8080/inferiorid");
        obj.addInferiorAddress(new Address("inferriorAddress1"));
        obj.addInferiorAddress(new Address("inferriorAddress2"));
        System.out.println(obj.toString());
        obj = new Enrol(obj.toXml());
        System.out.println(obj.toString());
    }
}
