
package org.act.services.transaction.messages;
import org.act.services.transaction.util.*;


public class InferiorState extends Message{
	     public static final String ACTIVE = Status.STATUS_ACTIVE;
    public static final String INACCESSIBLE = Status.STATUS_INACCESSIBLE;
    public static final String UNKNOWN = Status.STATUS_UNKNOWN;
    
    private String inferiorIdentifier_ = null;
    
    private String superiorIdentifier_ = null;
    
    private String status_ = Status.STATUS_ACTIVE;
    
    private boolean responseRequested_ = true;
    
    public InferiorState()
    {
        super("btp:inferior-state", Message.TYPE_INFERIOR_STATE);
    }
    
    public InferiorState(String stat)
    {
        super("btp:inferior-state", Message.TYPE_INFERIOR_STATE);
        status_ = stat;
    }
    
    public InferiorState(XElement el)
    {
        super(el, Message.TYPE_INFERIOR_STATE);
        XHashtable h = el.toHashtable();
        id_ = el.getAttribute("id");
        Object obj = h.getSingle("/btp:inferior-state/btp:inferior-identifier");
        if (obj != null)
        {
            inferiorIdentifier_ = obj.toString();
        }
        obj = h.getSingle("/btp:inferior-state/btp:superior-identifier");
        if (obj != null)
        {
            superiorIdentifier_ = obj.toString();
        }
        obj = h.getSingle("/btp:inferior-state/btp:status");
        if (obj != null)
        {
            status_ = obj.toString();
        }
        obj = h.getSingle("/btp:inferior-state/btp:response-requested");
        if (obj != null)
        {
            responseRequested_ = !obj.toString().equalsIgnoreCase("false");
        }
    }
    
    public void setInferiorIdentifier(String uri)
    {
        inferiorIdentifier_ = uri;
    }
    
    public String getInferiorIdentifier()
    {
        return inferiorIdentifier_;
    }
    
    public void setSuperiorIdentifier(String uri)
    {
        superiorIdentifier_ = uri;
    }
    
    public String getSuperiorIdentifier()
    {
        return superiorIdentifier_;
    }
    
    public void setStatus(String status)
    {
        status_ = status;
    }
    
    public String getStatus()
    {
        return status_;
    }
    
    public void setResponseRequested(boolean response)
    {
        responseRequested_ = response;
    }
    
    public boolean getResponseRequested()
    {
        return responseRequested_;
    }
    
    public XElement toXml()
    {
        XElement el = new XElement("btp:inferior-state");
        if (id_ != null)
        {
            el.addAttribute("id", id_);
        }
        el.addElement(
            new XElement("btp:inferior-identifier", inferiorIdentifier_));
        el.addElement(
            new XElement("btp:superior-identifier", superiorIdentifier_));
        el.addElement(
            new XElement("btp:status", status_));
        el.addElement(
            new XElement("btp:response-requested",
                         (responseRequested_ ? "true" : "false")));
        addQualifiers(el);
        addTargetAddress(el);
        addSenderAddress(el);
        return el;
    }
    public static void main(String args[])
    {
        System.out.println("===== Build inferior-state");
        InferiorState obj = new InferiorState();
        obj.setSuperiorIdentifier("http://www.myaddress.com:8080/superiorid");
        obj.setInferiorIdentifier("http://www.myaddress.com:8080/inferiorid");
        obj.setResponseRequested(false);
        obj.setStatus("inaccessible");
        System.out.println(obj.toString());
        System.out.println("===== Extract inferior-state");
        obj = new InferiorState(obj.toXml());
        System.out.println(obj.toString());
    }
}
