
package org.act.services.transaction.messages;
import java.util.*;
import org.act.services.transaction.util.*;

public class Message {
	
    public static final int TYPE_UNKNOWN = 0;
    public static final int TYPE_BEGIN = 1;
    public static final int TYPE_BEGUN = 2;
    public static final int TYPE_CANCEL_INFERIORS = 3;
    public static final int TYPE_CANCEL = 4;
    public static final int TYPE_CANCELLED = 5;
    public static final int TYPE_CANCEL_TRANSACTION = 6;
    public static final int TYPE_CONFIRMED = 7;
    public static final int TYPE_CONFIRM = 8;
    public static final int TYPE_CONFIRM_ONE_PHASE = 9;
    public static final int TYPE_CONFIRM_TRANSACTION = 10;
    public static final int TYPE_CONTEXT = 11;
    public static final int TYPE_CONTEXT_REPLY = 12;
    public static final int TYPE_CONTRADICTION = 13;
    public static final int TYPE_ENROL = 14;
    public static final int TYPE_ENROLLED = 15;
    public static final int TYPE_FAULT = 16;
    public static final int TYPE_HAZARD = 17;
    public static final int TYPE_INFERIOR_STATE = 18;
    public static final int TYPE_INFERIOR_STATUSES = 19;
    public static final int TYPE_PREPARED = 20;
    public static final int TYPE_PREPARE_INFERIORS = 21;
    public static final int TYPE_PREPARE = 22;
    public static final int TYPE_REDIRECT = 23;
    public static final int TYPE_REQUEST_INFERIOR_STATUSES = 24;
    public static final int TYPE_REQUEST_STATUS = 25;
    public static final int TYPE_RESIGNED = 26;
    public static final int TYPE_RESIGN = 27;
    public static final int TYPE_STATUS = 28;
    public static final int TYPE_SUPERIOR_STATE = 29;
    public static final int TYPE_TRANSACTION_CANCELLED = 30;
    public static final int TYPE_TRANSACTION_CONFIRMED = 31;
    public static final int TYPE_RELATED_GROUP = 100;
    public static final int TYPE_ACT_ADMIN = 200;
    
    public static final String EMPTY = "";
    
    int type_ = Message.TYPE_UNKNOWN;
    
    String name_;
    
    String id_ = null;
    
    Vector repladdr_ = null;
    
    Vector senderaddr_ = null;
    
    Vector qualifiers_ = null;
    
    String taddr_ = null;
    
    
    public Message(String name, int type)
    {
        name_ = name;
        type_ = type;
    }
    
    public Message(XElement el, int type)
    {
        type_ = type;
        name_ = el.getName();
        String nspace = el.getNamespace();
        if (nspace == null)
        {
            nspace = "";
        }
        else
        {
            nspace += ":";
        }
        XHashtable h = el.toHashtable();
                repladdr_ = pickAddrSet(h, "/" + name_ + "/" + nspace + "reply-address");
                senderaddr_ = pickAddrSet(h,
                                  "/" + name_ + "/" + nspace + "sender-address");
                Vector v = h.get("/" + name_ + "/" + nspace + "qualifiers");
                if (v != null)
        {
            qualifiers_ = new Vector();
            for (int i = 0; i < v.size(); i++)
            {
                qualifiers_.addElement(new Qualifier((XElement)v.elementAt(i)));
            }
        }
                taddr_ = pickString(h,
                            "/" + name_ + "/" + nspace +
                            "target-additional-information");
            }
    
    public String getName()
    {
        return name_;
    }
    
    public int getType()
    {
        return type_;
    }
    
    public Object getSingle(String path, boolean local)
    {
        return toXml().toHashtable(local).getSingle(path);
    }
    
    public Object getSingle(String path, int pos, boolean local)
    {
        return toXml().toHashtable(local).getSingle(path, pos);
    }
    
    public Vector get(String path, boolean local)
    {
        return toXml().toHashtable(local).get(path);
    }
    
    protected String pickString(XHashtable h, String path)
    {
        Object obj = h.getSingle(path);
        return (obj == null ? null : obj.toString());
    }
    
    protected Vector pickAddrSet(XHashtable h, String path)
    {
        Vector ret = null;
        Vector v = h.get(path);
        if (v != null)
        {
            ret = new Vector();
            for (int i = 0; i < v.size(); i++)
            {
                ret.addElement(new Address((XElement)v.elementAt(i)));
            }
        }
        return ret;
    }
    
    public void addReplyAddress(Address add)
    {
        if (repladdr_ == null)
        {
            repladdr_ = new Vector();
        }
        repladdr_.addElement(add);
    }
    
    public Vector getReplyAddresses()
    {
        return repladdr_;
    }
    
    public Address getReplyAddress(int pos)
    {
        if (repladdr_ == null || pos >= repladdr_.size())
        {
            return null;
        }
        else
        {
            return (Address)repladdr_.elementAt(pos);
        }
    }
    
    public void addSenderAddress(Address add)
    {
        if (senderaddr_ == null)
        {
            senderaddr_ = new Vector();
        }
        senderaddr_.addElement(add);
    }
    
    public Vector getSenderAddresses()
    {
        return senderaddr_;
    }
    
    public Address getSenderAddress(int pos)
    {
        if (senderaddr_ == null || pos >= senderaddr_.size())
        {
            return null;
        }
        else
        {
            return (Address)senderaddr_.elementAt(pos);
        }
    }
    
    public void addQualifier(Qualifier q)
    {
        if (qualifiers_ == null)
        {
            qualifiers_ = new Vector();
        }
        qualifiers_.addElement(q);
    }
    
    public Vector getQualifiers()
    {
        return qualifiers_;
    }
    
    public Qualifier getQualifier(int pos)
    {
        if (qualifiers_ == null || pos >= qualifiers_.size())
        {
            return null;
        }
        else
        {
            return (Qualifier)qualifiers_.elementAt(pos);
        }
    }
    
    public Qualifier getQualifier(String localname)
    {
        if (qualifiers_ == null)
        {
            return null;
        }
        for (int i = 0; i < qualifiers_.size(); i++)
        {
            Qualifier q = (Qualifier)qualifiers_.elementAt(i);
            if (q.getName().endsWith(localname))
            {
                return q;
            }
        }
        return null;
    }
    
    public void setTargetAddress(String i)
    {
        taddr_ = i;
    }
    public String getTargetAddress()
    {
        return taddr_;
    }
    
    public void setTransactionTimelimit(int seconds)
    {
        addQualifier(new TransactionTimelimit(seconds));
    }
    
    public int getTransactionTimelimit()
    {
        TransactionTimelimit q
            = (TransactionTimelimit)getQualifier("transaction-timelimit");
        return (q == null ? -1 : q.getTimelimit());
    }
    
    public void setInferiorTimeout(int seconds, int decision)
    {
        addQualifier(new InferiorTimeout(seconds, decision));
    }
    
    public int getInferiorTimeout()
    {
        InferiorTimeout q
            = (InferiorTimeout)getQualifier("inferior-timeout");
        return (q == null ? -1 : q.getTimeout());
    }
    
    public String getInferiorTimeoutDecision()
    {
        InferiorTimeout q
            = (InferiorTimeout)getQualifier("inferior-timeout");
        return (q == null ? null : q.getDecision());
    }
    
    public void setMinimumInferiorTimeout(int seconds, int decision)
    {
        addQualifier(new MinimumInferiorTimeout(seconds));
    }
    
    public int getMinimumInferiorTimeout()
    {
        MinimumInferiorTimeout q
            = (MinimumInferiorTimeout)getQualifier("minimum-inferior-timeout");
        return (q == null ? -1 : q.getMinimumTimeout());
    }
    
    public void setInferiorName(String name)
    {
        addQualifier(new InferiorName(name));
    }
    
    public String getInferiorName()
    {
        InferiorName q
            = (InferiorName)getQualifier("inferior-name");
        return (q == null ? null : q.getInferiorName());
    }
    
    public String toString()
    {
        return toXml().toString();
    }
    
    protected void addQualifiers(XElement el)
    {
        if (qualifiers_ != null)
        {
            XElement q = new XElement("btp:qualifiers");
            for (int i = 0; i < qualifiers_.size(); i++)
            {
                q.addElement(((Qualifier)qualifiers_.elementAt(i)).toXml());
            }
            el.addElement(q);
        }
    }
    
    protected void addTargetAddress(XElement el)
    {
        if (taddr_ != null)
        {
            el.addElement(
                new XElement("btp:target-additional-information", taddr_));
        }
    }
    
    protected void addReplyAddress(XElement el)
    {
        if (repladdr_ != null)
        {
            for (int i = 0; i < repladdr_.size(); i++)
            {
                XElement ra = new XElement("btp:reply-address");
                ra.addElement(((Address)repladdr_.elementAt(i)).toXml());
                el.addElement(ra);
            }
        }
    }
    
    protected void addSenderAddress(XElement el)
    {
        if (senderaddr_ != null)
        {
            for (int i = 0; i < senderaddr_.size(); i++)
            {
                XElement sender = new XElement("btp:sender-address");
                sender.addElement(((Address)senderaddr_.elementAt(i)).toXml());
                el.addElement(sender);
            }
        }
    }
    
    public XElement toXml()
    {
        XElement el = new XElement(name_);
        if (id_ != null)
        {
            el.addAttribute("id", id_);
        }
        addQualifiers(el);
        addTargetAddress(el);
        addReplyAddress(el);
        addSenderAddress(el);
        return el;
    }
    
    public String toHtml()
    {
        return BTPUtil.toHtml(toXml().toString());
    }
    public void setTransactionId(String txid)
    {
        id_ = txid;
    }
    public String getTransactionId()
    {
        return id_;
    }
    public static void main(String[] args)
    {
        Message message1 = new Message("test", Message.TYPE_BEGIN);
        message1.setTransactionId("1234556");
        String id = message1.getTransactionId();
    }
	
}
