
package org.act.services.transaction.messages;
import java.util.*;
import org.act.services.transaction.util.*;

public class RelatedGroup extends Message{
	public static final int RG_UNKNOWN = 0;
    public static final int RG_APP = 1;
    public static final int RG_BEGIN = 2;
    public static final int RG_BEGUN = 4;
    public static final int RG_CANCELLED = 8;
    public static final int RG_CONTEXT = 16;
    public static final int RG_CONTEXT_REPLY = 32;
    public static final int RG_ENROL = 64;
    public static final int RG_PREPARED = 128;
    public static final int RG_BEGIN_CONTEXT = RG_BEGIN | RG_CONTEXT;
    public static final int RG_BEGUN_CONTEXT = RG_BEGUN | RG_CONTEXT;
    public static final int RG_CONTEXT_APP = RG_CONTEXT | RG_APP;
    public static final int RG_CTXREPLY_APP = RG_CONTEXT_REPLY | RG_APP;
    
    int subtype_ = RelatedGroup.RG_UNKNOWN;
    
    Vector messages_ = null;
    
    public RelatedGroup()
    {
        super("btp:related-group", Message.TYPE_RELATED_GROUP);
    }
    
    public RelatedGroup(int subtype)
    {
        super("btp:related-group", Message.TYPE_RELATED_GROUP);
        subtype_ = subtype;
    }
    
    public RelatedGroup(Message m1)
    {
        super("btp:related-group", Message.TYPE_RELATED_GROUP);
        addMessage(m1);
    }
    
    public RelatedGroup(Message m1, Message m2)
    {
        this(m1);
        addMessage(m2);
    }
    
    public RelatedGroup(XElement el)
    {
        super(el, Message.TYPE_RELATED_GROUP);
        id_ = el.getAttribute("id");
        XVector v = el.getNodes();
        if (v != null)
        {
            for (int i = 0; i < el.getNodeCount(); i++)
            {
                try
                {
                    addMessage(MessageUtil.parseMessage(v.elementAt(i).toString()));
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
        }
    }
    
    public void addMessage(Message msg)
    {
        if (msg == null)
        {
            return;
        }
        if (messages_ == null)
        {
            messages_ = new Vector();
        }
        switch (msg.getType())
        {
            case Message.TYPE_BEGIN:
                subtype_ |= RelatedGroup.RG_BEGIN;
                break;
            case Message.TYPE_BEGUN:
                subtype_ |= RelatedGroup.RG_BEGUN;
                break;
            case Message.TYPE_CONTEXT:
                subtype_ |= RelatedGroup.RG_CONTEXT;
                break;
            case Message.TYPE_CONTEXT_REPLY:
                subtype_ |= RelatedGroup.RG_CONTEXT_REPLY;
                break;
            default:
                subtype_ |= RelatedGroup.RG_APP;
        }
        messages_.addElement(msg);
    }
    
    public Vector getMessages()
    {
        return messages_;
    }
    
    public Message getMessage(int pos)
    {
        if (messages_ == null || pos >= messages_.size())
        {
            return null;
        }
        return (Message)messages_.elementAt(pos);
    }
    
    public int getMessageCount()
    {
        return (messages_ == null ? 0 : messages_.size());
    }
    
    public Vector getMessagesByType(int type)
    {
        if (messages_ == null)
        {
            return null;
        }
        Vector ret = null;
        for (int i = 0; i < messages_.size(); i++)
        {
            Message m = (Message)messages_.elementAt(i);
            if (m.getType() == type)
            {
                if (ret == null)
                {
                    ret = new Vector();
                }
                ret.addElement(m);
            }
        }
        return ret;
    }
    
    public Message getMessageByType(int type)
    {
        if (messages_ == null)
        {
            return null;
        }
        for (int i = 0; i < messages_.size(); i++)
        {
            Message m = (Message)messages_.elementAt(i);
            if (m.getType() == type)
            {
                return m;
            }
        }
        return null;
    }
    
    public Begin getBegin()
    {
        return (Begin)getMessageByType(Message.TYPE_BEGIN);
    }
    
    public Begun getBegun()
    {
        return (Begun)getMessageByType(Message.TYPE_BEGUN);
    }
    public Context getContext()
    {
        return (Context)getMessageByType(Message.TYPE_CONTEXT);
    }
    public ContextReply getContextReply()
    {
        return (ContextReply)getMessageByType(Message.TYPE_CONTEXT_REPLY);
    }
    public int getSubtype()
    {
        return subtype_;
    }
    
    public XElement toXml()
    {
        XElement el = new XElement(name_);
        if (id_ != null)
        {
            el.addAttribute("id", id_);
        }
        for (int i = 0; i < messages_.size(); i++)
        {
            el.addElement(((Message)messages_.elementAt(i)).toXml());
        }
        return el;
    }
    public static void main(String[] args)
    {
        RelatedGroup relatedGroup1 = new RelatedGroup();
        Begin begin = new Begin();
        begin.setAtom(false);
        Context context = new Context("superioraddd", "superiorid1");
                relatedGroup1.addMessage(begin);
        relatedGroup1.addMessage(context);
        System.out.println(relatedGroup1.toString());
    }
}
