
package org.act.services.transaction.util;
import org.act.services.pool.IObjectPool;
import org.act.services.pool.ObjectPool;
import org.act.services.runtime.IRuntime;
import org.act.services.thread.ThreadPool;
import org.act.services.transaction.messages.ApplicationMessage;
import org.act.services.transaction.messages.Begin;
import org.act.services.transaction.messages.Begun;
import org.act.services.transaction.messages.Cancel;
import org.act.services.transaction.messages.CancelInferiors;
import org.act.services.transaction.messages.CancelTransaction;
import org.act.services.transaction.messages.Cancelled;
import org.act.services.transaction.messages.Confirm;
import org.act.services.transaction.messages.ConfirmOnePhase;
import org.act.services.transaction.messages.ConfirmTransaction;
import org.act.services.transaction.messages.Confirmed;
import org.act.services.transaction.messages.Context;
import org.act.services.transaction.messages.ContextReply;
import org.act.services.transaction.messages.Contradiction;
import org.act.services.transaction.messages.Enrol;
import org.act.services.transaction.messages.Enrolled;
import org.act.services.transaction.messages.Fault;
import org.act.services.transaction.messages.Hazard;
import org.act.services.transaction.messages.InferiorState;
import org.act.services.transaction.messages.InferiorStatuses;
import org.act.services.transaction.messages.Message;
import org.act.services.transaction.messages.Prepare;
import org.act.services.transaction.messages.PrepareInferiors;
import org.act.services.transaction.messages.Prepared;
import org.act.services.transaction.messages.Redirect;
import org.act.services.transaction.messages.RelatedGroup;
import org.act.services.transaction.messages.RequestInferiorStatuses;
import org.act.services.transaction.messages.RequestStatus;
import org.act.services.transaction.messages.Resign;
import org.act.services.transaction.messages.Resigned;
import org.act.services.transaction.messages.Status;
import org.act.services.transaction.messages.SuperiorState;
import org.act.services.transaction.messages.TransactionCancelled;
import org.act.services.transaction.messages.TransactionConfirmed;

public class MessageUtil {
	
		private static IObjectPool msgIn = null;
	private static IObjectPool msgOut = null;
	private static IRuntime runtime = null;
	
	private static ThreadPool threadPool;
	
	public MessageUtil(IObjectPool msgOut,ThreadPool pool,IRuntime runtime){
						this.msgIn=new ObjectPool();
		this.msgOut=msgOut;
		this.runtime=runtime;
		this.threadPool=pool;
	}
    public static Message parseMessage(XElement el)
    {
        Message msg = null;
        String elname = el.getLocalName();
        if ("BEGIN".equalsIgnoreCase(elname))
        {
            msg = new Begin(el);
        }
        else if ("BEGUN".equalsIgnoreCase(elname))
        {
            msg = new Begun(el);
        }
        else if ("CANCEL-INFERIORS".equalsIgnoreCase(elname))
        {
            msg = new CancelInferiors(el);
        }
        else if ("CANCEL".equalsIgnoreCase(elname))
        {
            msg = new Cancel(el);
        }
        else if ("CANCELLED".equalsIgnoreCase(elname))
        {
            msg = new Cancelled(el);
        }
        else if ("CANCEL-TRANSACTION".equalsIgnoreCase(elname))
        {
            msg = new CancelTransaction(el);
        }
        else if ("CONFIRMED".equalsIgnoreCase(elname))
        {
            msg = new Confirmed(el);
        }
        else if ("CONFIRM".equalsIgnoreCase(elname))
        {
            msg = new Confirm(el);
        }
        else if ("CONFIRM-ONE-PHASE".equalsIgnoreCase(elname))
        {
            msg = new ConfirmOnePhase(el);
        }
        else if ("CONFIRM-TRANSACTION".equalsIgnoreCase(elname))
        {
            msg = new ConfirmTransaction(el);
        }
        else if ("CONTEXT".equalsIgnoreCase(elname))
        {
            msg = new Context(el);
        }
        else if ("CONTEXT-REPLY".equalsIgnoreCase(elname))
        {
            msg = new ContextReply(el);
        }
        else if ("CONTRADICTION".equalsIgnoreCase(elname))
        {
            msg = new Contradiction(el);
        }
        else if ("ENROL".equalsIgnoreCase(elname))
        {
            msg = new Enrol(el);
        }
        else if ("ENROLLED".equalsIgnoreCase(elname))
        {
            msg = new Enrolled(el);
        }
        else if ("FAULT".equalsIgnoreCase(elname))
        {
            msg = new Fault(el);
        }
        else if ("HAZARD".equalsIgnoreCase(elname))
        {
            msg = new Hazard(el);
        }
        else if ("INFERIOR-STATE".equalsIgnoreCase(elname))
        {
            msg = new InferiorState(el);
        }
        else if ("INFERIOR-STATUSES".equalsIgnoreCase(elname))
        {
            msg = new InferiorStatuses(el);
        }
        else if ("PREPARED".equalsIgnoreCase(elname))
        {
            msg = new Prepared(el);
        }
        else if ("PREPARE-INFERIORS".equalsIgnoreCase(elname))
        {
            msg = new PrepareInferiors(el);
        }
        else if ("PREPARE".equalsIgnoreCase(elname))
        {
            msg = new Prepare(el);
        }
        else if ("REDIRECT".equalsIgnoreCase(elname))
        {
            msg = new Redirect(el);
        }
        else if ("REQUEST-INFERIOR-STATUSES".equalsIgnoreCase(elname))
        {
            msg = new RequestInferiorStatuses(el);
        }
        else if ("REQUEST-STATUS".equalsIgnoreCase(elname))
        {
            msg = new RequestStatus(el);
        }
        else if ("RESIGNED".equalsIgnoreCase(elname))
        {
            msg = new Resigned(el);
        }
        else if ("RESIGN".equalsIgnoreCase(elname))
        {
            msg = new Resign(el);
        }
        else if ("STATUS".equalsIgnoreCase(elname))
        {
            msg = new Status(el);
        }
        else if ("SUPERIOR-STATE".equalsIgnoreCase(elname))
        {
            msg = new SuperiorState(el);
        }
        else if ("TRANSACTION-CANCELLED".equalsIgnoreCase(elname))
        {
            msg = new TransactionCancelled(el);
        }
        else if ("TRANSACTION-CONFIRMED".equalsIgnoreCase(elname))
        {
            msg = new TransactionConfirmed(el);
        }
        else if ("RELATED-GROUP".equalsIgnoreCase(elname))
        {
            msg = new RelatedGroup(el);
        }
        else
        {
            msg = new ApplicationMessage(el);
        }
        return msg;
    }
    
    public static Message parseMessage(String xmlMsg)
        throws Exception
    {
        if (xmlMsg == null || xmlMsg.length() < 1)
        {
            return null;
        }
        XElement el = BTPUtil.changeXElement(xmlMsg);
        Message msg = parseMessage(el);
        return msg;
    }
    
    
    
    public static void sendMessage(String urlPoint, Message message,String txid)
        throws Exception
    {
        
    }
    
    public static void sendEnrolMessage(String urlPoint, Message message)
    throws Exception
{
    
}
        public static void PutMessage(Object obj){
        msgIn.put(obj);
    }
        public static void main(String[] args)
    {
            }
	
}
