
package org.act.services.workflow.context;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Stack;
import org.act.metaset.Metaset;
import org.act.services.pool.IObjectPool;
import org.act.services.runtime.Application;
import org.act.services.runtime.IRuntime;
import org.act.services.workflow.activity.basic.AssignImpl;
import org.act.services.workflow.activity.basic.EmptyImpl;
import org.act.services.workflow.activity.basic.InvokeImpl;
import org.act.services.workflow.activity.basic.ReceiveImpl;
import org.act.services.workflow.activity.basic.ReplyImpl;
import org.act.services.workflow.activity.basic.ThrowImpl;
import org.act.services.workflow.activity.basic.WaitImpl;
import org.act.services.workflow.activity.extension.BusinessTransactionImpl;
import org.act.services.workflow.activity.extension.ScopeOverImpl;
import org.act.services.workflow.activity.scope.CompensateImpl;
import org.act.services.workflow.activity.scope.ScopeImpl;
import org.act.services.workflow.activity.scope.TerminateImpl;
import org.act.services.workflow.activity.structured.FlowImpl;
import org.act.services.workflow.activity.structured.PickImpl;
import org.act.services.workflow.activity.structured.SequenceImpl;
import org.act.services.workflow.activity.structured.SwitchImpl;
import org.act.services.workflow.activity.structured.WhileImpl;
import org.act.services.workflow.debug.Debug;

public class WorkflowInstance {
	private WorkflowRuntimeContext rc = null;
		private IObjectPool msgIn;
	private IRuntime runtime;
	
	public WorkflowInstance(WorkflowRuntimeContext rc, IObjectPool msgIn, IRuntime runtime) {
		this.rc = rc;
		this.msgIn = msgIn;
		this.runtime = runtime;
		if(Debug.isDebug()){
			Debug.setRc(this.rc);
		}
	}
	public void run() {
		String instanceID = this.rc.getInstanceID();
		String type = this.rc.getWc().getName();
		String status = "running";
		String beginTime = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS"))
				.format(new GregorianCalendar().getTime());
		this.rc.getWir().setInstanceID(instanceID);
		this.rc.getWir().setType(type);
		this.rc.getWir().setStatus(status);
		this.rc.getWir().setBeginTime(beginTime);       	
		
		while (!rc.getActivitiesStack().isEmpty()) {
			Metaset activity = (Metaset) rc.getActivitiesStack().pop();
			String name = activity.getName();
			invokeFromName(name, activity, rc.getActivitiesStack());
		}
		
		status = "completed";
		String finishTime = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS"))
				.format(new GregorianCalendar().getTime());
		this.rc.getWir().setStatus(status);
		this.rc.getWir().setFinishTime(finishTime);
	}
	private void invokeFromName(String name, Metaset activity, Stack stack) {
		if (name.equals("receive")) {
			BPELreceive(activity, stack);
		} else if (name.equals("sequence")) {
			BPELsequence(activity, stack);
		} else if (name.equals("switch")) {
			BPELswitch(activity, stack);
		} else if (name.equals("while")) {
			BPELwhile(activity, stack);
		} else if (name.equals("reply")) {
			BPELreply(activity, stack);
		} else if (name.equals("invoke")) {
			BPELinvoke(activity, stack);
		} else if (name.equals("assign")) {
			BPELassign(activity, stack);
		} else if (name.equals("throw")) {
			BPELthrow(activity, stack);
		} else if (name.equals("terminate")) {
			BPELterminate(activity, stack);
		} else if (name.equals("wait")) {
			BPELwait(activity, stack);
		} else if (name.equals("empty")) {
			BPELempty(activity, stack);
		} else if (name.equals("pick")) {
			BPELpick(activity, stack);
		} else if (name.equals("flow")) {
			BPELflow(activity, stack);
		} else if (name.equals("scope")) {
			BPELscope(activity, stack);
		} else if (name.equals("compensate")) {
			BPELcompensate(activity, stack);
		} else if (name.equals("scopeOver")) {
			BPELscopeOver(activity, stack);
		} else if (name.equals("concurrentActivities")) {
			doConcurrentActivities(activity, stack);
		} else if (name.equals("businessTransaction")) {
			BPELbusinessTransaction(activity, stack);
		}
	}
	public void stepRun(Stack stack) {
		Metaset activity = (Metaset) stack.pop();
		String name = activity.getName();
		invokeFromName(name, activity, stack);
	}
	public void dealSource(Metaset activity) {
		String linkName = activity.selector(".\\", "linkName");
		if (Application.isDebug()) {
			System.out.println("Workflow.dealSource():" + linkName);
		}
		Metaset ca = getConcurrentActivities(activity);
		if (ca != null) {
			Metaset sources = ca.filter(".\\sources");
			if (sources == null || sources.isEmpty()) {
				Metaset ss = new Metaset("sources");
				ca.add(ss);
				ss.add(activity);
			} else {
				((Metaset) sources.getChilds().get(0)).add(activity);
			}
		} else {
					}
	}
	private Metaset getConcurrentActivities(Metaset activity) {
		Metaset father = activity.getFather();
		while (father != null) {
						if (father.getName().equals("concurrentActivities")) {
				return father;
			}
			father = father.getFather();
		}
		return null;
	}
	public void doConcurrentActivities(Metaset activity, Stack stack) {
		Metaset flows = activity.filter(".\\concurrentFlow");
		for (int i = 0; i < flows.getChilds().size(); i++) {
			Metaset flow = (Metaset) flows.getChilds().get(i);
			if (!flow.isEmpty()) {
				Stack assistantStack = new Stack();
								Metaset firstChildActivity = (Metaset) flow.getChilds()
						.get(0);
								Metaset targets = firstChildActivity.filter(".\\target");
				boolean pushActivity = false;
				if (targets == null || targets.isEmpty()) {
										pushActivity = true;
				} else {
															boolean result = true;
					for (int j = 0; j < targets.getChilds().size(); j++) {
						Metaset target = (Metaset) targets.getChilds()
								.get(j);
						String linkName = target.selector(".\\", "linkName");
						Metaset ss = activity.filter(".\\sources\\source",
								"linkName", linkName);
						if (ss == null || ss.isEmpty()) {
							result = false;
							break;
						}
					}
					if (result) {
						firstChildActivity.remove(".\\target");
												pushActivity = true;
					} else {
												continue;
					}
				}
				if (pushActivity) {
										Metaset sources = firstChildActivity
							.filter(".\\source");
					if (sources != null) {
						for (int j = 0; j < sources.getChilds().size(); j++) {
							assistantStack.push(sources.getChilds().get(j));
						}
					}
					firstChildActivity.remove(".\\source");
					assistantStack.push(firstChildActivity);
										stepRun(assistantStack);
										flow.getChilds().remove(0);
					List activities = new ArrayList();
					List source = new ArrayList();
					while (!assistantStack.isEmpty()) {
						Metaset ss = (Metaset) assistantStack.pop();
						if (ss.getName().equals("source")) {
							source.add(ss);
						} else {
							activities.add(ss);
						}
					}
										flow.getChilds().addAll(0, activities);
										flow.getChilds().addAll(source);
				}
			}
						if (flow.isEmpty()) {
				flow.getFather().remove(flow);
			} else {
				boolean result = true;
				for (int j = 0; j < flow.getChilds().size(); j++) {
					Metaset source = (Metaset) flow.getChilds().get(j);
					if (!source.getName().equals("source")) {
						result = false;
						break;
					}
				}
				if (result) {
					for (int j = 0; j < flow.getChilds().size(); j++) {
						Metaset source = (Metaset) flow.getChilds()
								.get(j);
						dealSource(source);
					}
				}
			}
		}
		for (int i = 0; i < flows.getChilds().size();) {
			Metaset flow = (Metaset) flows.getChilds().get(i);
			if (flow.getChilds().size() == 0) {
				flows.remove(flow);
			} else {
				i++;
			}
		}
				if (flows.getChilds().size() > 0) {
			stack.push(activity);
		}
	}
	public void BPELreceive(Metaset activity, Stack stack) {
		ReceiveImpl receiveImpl = new ReceiveImpl(activity, this.rc, stack,
				msgIn);
		receiveImpl.process();
	}
	public void BPELreply(Metaset activity, Stack stack) {
		ReplyImpl replyImpl = new ReplyImpl(activity, this.rc, stack, runtime);
		replyImpl.process();
	}
	public void BPELinvoke(Metaset activity, Stack stack) {
		InvokeImpl invokeImpl = new InvokeImpl(activity, this.rc, stack, msgIn,
				runtime);
		invokeImpl.process();
	}
	public void BPELassign(Metaset activity, Stack stack) {
		AssignImpl assignImpl = new AssignImpl(activity, this.rc, stack);
		assignImpl.process();
	}
	public void BPELthrow(Metaset activity, Stack stack) {
		ThrowImpl throwImpl = new ThrowImpl(activity, this.rc, stack);
		throwImpl.process();
	}
	public void BPELterminate(Metaset activity, Stack stack) {
		TerminateImpl terminateImpl = new TerminateImpl(activity, this.rc,
				stack);
		terminateImpl.process();
	}
	public void BPELwait(Metaset activity, Stack stack) {
		WaitImpl waitImpl = new WaitImpl(activity, this.rc, stack);
		waitImpl.process();
	}
	public void BPELempty(Metaset activity, Stack stack) {
		EmptyImpl emptyImpl = new EmptyImpl(activity, this.rc, stack);
		emptyImpl.process();
	}
	public void BPELsequence(Metaset activity, Stack stack) {
		SequenceImpl sequenceImpl = new SequenceImpl(activity, this.rc, stack);
		sequenceImpl.process();
	}
	public void BPELswitch(Metaset activity, Stack stack) {
		SwitchImpl switchImpl = new SwitchImpl(activity, this.rc, stack);
		switchImpl.process();
	}
	public void BPELwhile(Metaset activity, Stack stack) {
		WhileImpl whileImpl = new WhileImpl(activity, this.rc, stack);
		whileImpl.process();
	}
	public void BPELpick(Metaset activity, Stack stack) {
		PickImpl pickImpl = new PickImpl(activity, this.rc, stack, msgIn);
		pickImpl.process();
	}
	public void BPELflow(Metaset activity, Stack stack) {
		FlowImpl flowImpl = new FlowImpl(activity, this.rc, stack);
		flowImpl.process();
	}
	public void BPELscope(Metaset activity, Stack stack) {
		ScopeImpl scopeImpl = new ScopeImpl(activity, this.rc, stack);
		scopeImpl.process();
	}
	public void BPELscopeOver(Metaset activity, Stack stack) {
		ScopeOverImpl scopeOverImpl = new ScopeOverImpl(activity, this.rc,
				stack);
		scopeOverImpl.process();
	}
	public void BPELcompensate(Metaset activity, Stack stack) {
		CompensateImpl compensateImpl = new CompensateImpl(activity, this.rc,
				stack);
		compensateImpl.process();
	}
	public void BPELbusinessTransaction(Metaset activity, Stack stack) {
		BusinessTransactionImpl businessTransactionImpl = new BusinessTransactionImpl(
				activity, this.rc, stack);
		businessTransactionImpl.process();
	}
	public void eventHandlers(Metaset activity) {
	}
}
