
package org.act.wsrf.resourceproperties;
import java.util.Calendar;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Timer;
import java.util.logging.Logger;
import org.act.metaset.Attribute;
import org.act.metaset.QName;
import org.act.metaset.Metaset;
import org.act.metaset.utils.MetasetUtil;
import org.act.wsrf.basefaults.BaseFault;
import org.act.wsrf.basefaults.Description;
import org.act.wsrf.resourceproperties.faults.InvalidResourcePropertyQName;
import org.act.wsrf.resourceproperties.faults.ResourceNotDestroyedFault;
import org.act.wsrf.resourceproperties.faults.ResourceUnknownFault;
import org.act.wsrf.resourceproperties.faults.UnableToModifyResourceProperty;
import org.act.wsrf.resourceproperties.lifetime.TerminationTimerTask;

public class Resource {
	
	private String id = null;
	private Metaset resource = null;
	
	public static Hashtable resourcemap = new Hashtable();
	
	public Resource(){
		
	}
	
	public Resource(String id){
		this.id = id;
	}
	
	public Resource(String id,Metaset resource){
		this.id = id;
		this.resource = resource;
		resourcemap.put(id,resource);
	}
	
	public int getSize(){
		return resourcemap.size();
	}	
	
	public Metaset getResource(String id){
		if(resourcemap.isEmpty()){
			Logger logger = Logger.getLogger("Resource map is empty!");
		}
		
		if(!resourcemap.containsKey(id)){
			Logger logger = Logger.getLogger("No such Resource");
		}
		
		Metaset resource = (Metaset)resourcemap.get(id);
		return resource;
	}
	
	public void addResource(Metaset resource,String id){
		resourcemap.put(id,resource);
	}
	
	public Metaset getResourceProperty(QName name,String id)throws ResourceUnknownFault,InvalidResourcePropertyQName{
		Metaset resource = getResource(id);
		
		if(resource==null){
					BaseFault baseFault = new BaseFault("ResourceUnownFault");
			Description des = new Description("Description");
			des.setValue("The service cann't recoginize this resource");
			ResourceUnknownFault fault = new ResourceUnknownFault(baseFault);
			fault.setDescription(des);
			throw fault;
		}
		
		Metaset result = new Metaset("wsrp","GetResourcePropertyResponse");
		
		List elements = resource.getChilds();
		Iterator iter = elements.iterator();
		while(iter.hasNext()){
			Metaset element = (Metaset)iter.next();
						
			if((element.getName().equals(name.getName()))&&(MetasetUtil.getActualNamespace(element,element.getNamespace()).equals(name.getNameSpace()))){
								if(name.getName().equals("CurrentTime")){
					Calendar time = Calendar.getInstance();
					element.setValue(time.getTime().toLocaleString());
				}
				result.add(element);
				return result;
			}
		}
		
		if(result.getChilds().size()<=0){
						BaseFault baseFault = new BaseFault("InvalidResourcePropertyQName");
			Description des = new Description("Description");
			des.setValue("The WS-Resource dosen't include the property represented by the QName "+name.toString());
			InvalidResourcePropertyQName fault = new InvalidResourcePropertyQName(baseFault);
			fault.setDescription(des);
			throw fault;
		}
		return null;
	}
	
	public Metaset getMultipleResourceProperties(QName[] name,String id)throws ResourceUnknownFault,InvalidResourcePropertyQName{
		Metaset resource = getResource(id);
		
		if(resource==null){
			BaseFault baseFault = new BaseFault("ResourceUnownFault");
			Description des = new Description("Description");
			des.setValue("The service cann't recoginize this resource");
			ResourceUnknownFault fault = new ResourceUnknownFault(baseFault);
			fault.setDescription(des);
			throw fault;
		}
		
		Metaset result = new Metaset("wsrp","GetMultipleResourcePropertiesResponse");
		for(int i=0;i<name.length;i++){
			List elements = resource.getChilds();
			Iterator iter = elements.iterator();
			Metaset faultSet = null;
			while(iter.hasNext()){
				Metaset element = (Metaset)iter.next();
								if((element.getName().equals(name[i].getName()))&&(MetasetUtil.getActualNamespace(element,element.getNamespace()).equals(name[i].getNameSpace()))){
										if(name[i].getName().equals("CurrentTime")){
						Calendar time = Calendar.getInstance();
						element.setValue(time.getTime().toLocaleString());
					}
					faultSet = element;
					result.add(element);
				}
			}
			if(faultSet==null){
								BaseFault baseFault = new BaseFault("InvalidResourcePropertyQName");
				Description des = new Description("Description");
				des.setValue("The WS-Resource dosen't include the property represented by the QName "+name[i].toString());
				InvalidResourcePropertyQName fault = new InvalidResourcePropertyQName(baseFault);
				fault.setDescription(des);
				throw fault;
			}
		}
		
		return result;
	}
	
	
	public Metaset insert(Metaset source,String id)throws ResourceUnknownFault{
		Metaset resource = getResource(id);
		if(resource==null){
						BaseFault baseFault = new BaseFault("ResourceUnownFault");
			Description des = new Description("Description");
			des.setValue("The service cann't recoginize this resource");
			ResourceUnknownFault fault = new ResourceUnknownFault(baseFault);
			fault.setDescription(des);
			throw fault;
		}
		
				if(source.getChilds().size()>0){
			
			List elements = source.getChilds();
			Iterator iter = elements.iterator();
			while(iter.hasNext()){
				Metaset element = (Metaset)iter.next();
				
								String actualNamespace = MetasetUtil.getActualNamespace(element,element.getNamespace());
				if(actualNamespace!=null && actualNamespace!=""){
					List attributeList = resource.getAttributes();
										boolean equal = false;
					Iterator attrIter = attributeList.iterator();
					while(attrIter.hasNext()){
						Attribute attribute = (Attribute)attrIter.next();
						if(attribute.getName().equals(source.getNamespace())){
							equal = true;
						}
					}
										if(equal == false){
												String attrName = element.getNamespace();
						Attribute attrAdd = null;
												if(attrName!=""){
							attrAdd = new Attribute(new QName("xmlns",attrName),actualNamespace);
						}
						else{
							attrAdd = new Attribute(new QName("xmlns"),actualNamespace);
						}
						resource.add(attrAdd);	
					}
										else{
												String prefixName = element.getNamespace()+"add";
						source.setNamespace(prefixName);
												Attribute attrAdd = new Attribute(new QName("xmlns",prefixName),actualNamespace);
						resource.add(attrAdd);
					}
					
				}
				resource.add(element);
			}
		}
				else{
			
						String actualNamespace = MetasetUtil.getActualNamespace(source,source.getNamespace());
			if(actualNamespace!=null && actualNamespace!=""){
				
				List attributeList = resource.getAttributes();
								boolean equal = false;
				Iterator attrIter = attributeList.iterator();
				while(attrIter.hasNext()){
					Attribute attribute = (Attribute)attrIter.next();
					if(attribute.getName().equals(source.getNamespace())){
						equal = true;
					}
				}
								if(equal == false){
										String attrName = source.getNamespace();
					Attribute attrAdd = null;
										if(attrName!=""){
						attrAdd = new Attribute(new QName("xmlns",attrName),actualNamespace);
					}
					else{
						attrAdd = new Attribute(new QName("xmlns"),actualNamespace);
					}
					resource.add(attrAdd);
										List attrList = source.getAttributes();
					Attribute[] attrArray = new Attribute[attrList.size()];
					Iterator iter = attrList.iterator();
					int count = 0;
					while(iter.hasNext()){
						Attribute at = (Attribute)iter.next();
						attrArray[count++] = at;
					}
					for(int i=0;i<attrList.size();i++){
						source.remove(attrArray[i]);
					}	
				}
								else{
										String prefixName = source.getNamespace()+"add";
					source.setNamespace(prefixName);
										Attribute attrAdd = new Attribute(new QName("xmlns",prefixName),actualNamespace);
					resource.add(attrAdd);
										List attrList = source.getAttributes();
					Attribute[] attrArray = new Attribute[attrList.size()];
					Iterator iter = attrList.iterator();
					int count = 0;
					while(iter.hasNext()){
						Attribute at = (Attribute)iter.next();
						attrArray[count++] = at;
					}
					for(int i=0;i<attrList.size();i++){
						source.remove(attrArray[i]);
					}	
				}
				
			}
			resource.add(source);
		}
		return resource;
	}
	
	
	public Metaset delete(QName name,String id)throws ResourceUnknownFault,UnableToModifyResourceProperty,InvalidResourcePropertyQName{
		Metaset resource = getResource(id);
		if(resource==null){
						BaseFault baseFault = new BaseFault("ResourceUnownFault");
			Description des = new Description("Description");
			des.setValue("The service cann't recoginize this resource");
			ResourceUnknownFault fault = new ResourceUnknownFault(baseFault);
			fault.setDescription(des);
			throw fault;
		}
				if(name.getName().equals("TerminationTime")||name.getName().equals("CurrentTime")){
						BaseFault baseFault = new BaseFault("UnableToModifyResourceProperty");
			Description des = new Description("Description");
			des.setValue("The property that going to be modified is read-only or can not be modified");
			UnableToModifyResourceProperty fault = new UnableToModifyResourceProperty(baseFault);
			fault.setDescription(des);
			throw fault;
		}
		
		Metaset delelement = null;
		List elements = resource.getChilds();
		Iterator iter = elements.iterator();
		while(iter.hasNext()){
			Metaset element = (Metaset)iter.next();
			if(element.getName().equals("TerminationTime") || element.getName().equals("CurrentTime")){
				continue;
			}
						if((element.getName().equals(name.getName()))&&(MetasetUtil.getActualNamespace(element,element.getNamespace()).equals(name.getNameSpace()))){
				delelement = element;
			}
		}
		if(delelement==null){
						BaseFault baseFault = new BaseFault("InvalidResourcePropertyQName");
			Description des = new Description("Description");
			des.setValue("The WS-Resource dosen't include the property represented by the QName "+name.toString());
			InvalidResourcePropertyQName fault = new InvalidResourcePropertyQName(baseFault);
			fault.setDescription(des);
			throw fault;
		}
		
		resource.remove(delelement);
		return resource;
	}
	
	
	public Metaset update(Metaset source,String id)throws ResourceUnknownFault,InvalidResourcePropertyQName{
		Metaset resource = getResource(id);
		if(resource==null){
						BaseFault baseFault = new BaseFault("ResourceUnownFault");
			Description des = new Description("Description");
			des.setValue("The service cann't recoginize this resource");
			ResourceUnknownFault fault = new ResourceUnknownFault(baseFault);
			fault.setDescription(des);
			throw fault;
		}
				if(source.getChilds().size()==0){
			List elements = resource.getChilds();
			Iterator iter = elements.iterator();
			Metaset faultSet = null;
			while(iter.hasNext()){
				Metaset element = (Metaset)iter.next();
								if((source.getName().equals(element.getName()))&&(MetasetUtil.getActualNamespace(source,source.getNamespace()).equals(MetasetUtil.getActualNamespace(element,element.getNamespace())))){
					element.setValue(source.getValue());
					faultSet = element;
				}
			}
			if(faultSet==null){
								BaseFault baseFault = new BaseFault("InvalidResourcePropertyQName");
				Description des = new Description("Description");
				des.setValue("The WS-Resource dosen't include the property represented by the QName "+source.toString());
				InvalidResourcePropertyQName fault = new InvalidResourcePropertyQName(baseFault);
				fault.setDescription(des);
				throw fault;
			}
		}
				else{
			List sourcelist = source.getChilds();
			Iterator sourceIter = sourcelist.iterator();
			while(sourceIter.hasNext()){
				Metaset sourceelement = (Metaset)sourceIter.next();
				List elements = resource.getChilds();
				Iterator iter = elements.iterator();
				Metaset faultSet = null;
				while(iter.hasNext()){
					Metaset element = (Metaset)iter.next();
										if((sourceelement.getName().equals(element.getName()))&&(MetasetUtil.getActualNamespace(sourceelement,sourceelement.getNamespace()).equals(MetasetUtil.getActualNamespace(element,element.getNamespace())))){
						element.setValue(sourceelement.getValue());
						faultSet = element;
					}
				}
				if(faultSet==null){
										BaseFault baseFault = new BaseFault("InvalidResourcePropertyQName");
					Description des = new Description("Description");
					des.setValue("The WS-Resource dosen't include the property represented by the QName "+sourceelement.toString());
					InvalidResourcePropertyQName fault = new InvalidResourcePropertyQName(baseFault);
					fault.setDescription(des);
					throw fault;
				}
			}
		}
		return resource;
	}
	
	
	public Metaset destroy(String id)throws ResourceUnknownFault,ResourceNotDestroyedFault{
				boolean destroySucc = false;
		
		Metaset resource = getResource(id);
				if(resource==null){
						BaseFault baseFault = new BaseFault("ResourceUnownFault");
			Description des = new Description("Description");
			des.setValue("The service cann't recoginize this resource");
			ResourceUnknownFault fault = new ResourceUnknownFault(baseFault);
			fault.setDescription(des);
			throw fault;
		}
		
		if(resource!=null){
			resourcemap.remove(id);
			destroySucc = true;
		}
		
				
		Metaset response = new Metaset("wsrl","DestroyResponse");
		
				if(destroySucc==true){
			return response;
		}
		else{
						BaseFault baseFault = new BaseFault("ResourceNotDestroyedFault");
			Description des = new Description("Description");
			des.setValue("This Resource cann't destroy");
			ResourceNotDestroyedFault fault = new ResourceNotDestroyedFault(baseFault);
			fault.setDescription(des);
			throw fault;
		}
	}
	
	
	public Metaset setTerminationTime(String id,Calendar newTerminationTime)throws ResourceUnknownFault{
		Metaset resource = getResource(id);
		if(resource==null){
						BaseFault baseFault = new BaseFault("ResourceUnownFault");
			Description des = new Description("Description");
			des.setValue("The service cann't recoginize this resource");
			ResourceUnknownFault fault = new ResourceUnknownFault(baseFault);
			fault.setDescription(des);
			throw fault;
		}
				Calendar currentTime = Calendar.getInstance();
		Calendar newTermTime = null;
		
				if(newTerminationTime.before(currentTime)){
			resourcemap.remove(id);
			newTermTime = currentTime;
		}
		
				Timer myTimer = new Timer();
		TerminationTimerTask myTask = new TerminationTimerTask(id);
		myTimer.schedule(myTask,newTerminationTime.getTime());
		
				List childs = resource.getChilds();
		Iterator iter = childs.iterator();
		while(iter.hasNext()){
			Metaset element = (Metaset)iter.next();
			if(element.getName().equals("TerminationTime")){
								element.setValue(newTerminationTime.getTime().toLocaleString());
				newTermTime = newTerminationTime;
			}
		}
		
				
		Metaset response = new Metaset("wsrl","SetTerminationTimeResponse");
		Metaset newTTime = new Metaset("wsrl","NewTermintionTime");
		newTTime.setValue(newTermTime.getTime().toLocaleString());
		Metaset curTime = new Metaset("wsrl","CurrentTime");
		curTime.setValue(currentTime.getTime().toLocaleString());
		response.add(newTTime);
		response.add(curTime);
		
		return response;	
	}
	
	
		
			
		
										
									
	
	 public Calendar getCurrentTime(String id){
	 	Metaset resource = getResource(id);
		
	 	Calendar time = null;
	 	if(resource!=null){
	 		time = Calendar.getInstance();
	 	}
		
				return time;
	 }
	
	 
	public static void main(String[] args){
		
				System.out.println("һԴ");
		Metaset re = new Metaset("GenericDiskDriveProperties");
		re.add(new Attribute(new QName("xmlns","tns"),"http://example.com/diskDrive"));
		re.add(new Attribute(new QName("xmlns","wsrl"),"http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceLifetime-1.2-draft-01.xsd"));
		Metaset number = new Metaset("tns","NumberOfBlocks");
		number.setValue("22");
		Metaset blocksize = new Metaset("tns","BlockSize");
		blocksize.setValue("1024");
		Metaset manufacture = new Metaset("tns","Manufacturer");
		manufacture.setValue("IBM");
		Metaset terminationTime = new Metaset("wsrl","TerminationTime");
		terminationTime.setValue("2005-3-15 11:00");
		Metaset currentTime = new Metaset("wsrl","CurrentTime");
		currentTime.setValue("2005-3-15 11:00:00");
		re.add(number);
		number.setFather(re);
		re.add(blocksize);
		blocksize.setFather(re);
		re.add(manufacture);
		manufacture.setFather(re);
		re.add(terminationTime);
		terminationTime.setFather(re);
		re.add(currentTime);
		currentTime.setFather(re);
		System.out.println(re.toXML());
		
		String id = "12345";
		
		Resource resource = new Resource();
		resource.addResource(re,id);
		
				System.out.println("ȡĳ");
		QName name = new QName("http://example.com/diskDrive","NumberOfBlocks");
				Metaset result = null;
		try{
			result = resource.getResourceProperty(name,id);
		}catch(ResourceUnknownFault e){
			System.out.println(e.toString());
		}catch(InvalidResourcePropertyQName e){
			System.out.println(e.toString());
		}
		System.out.println(result.toXML());
		
				System.out.println("ȡ");
		QName name1  = new QName("http://example.com/diskDrive","BlockSize");
		QName[] names = new QName[2];
		names[0] = name;
		names[1] = name1;
		Metaset results = null;
		try{
			results = resource.getMultipleResourceProperties(names,id);
			
		}catch(ResourceUnknownFault e){
			System.out.println(e.toString());
		}catch(InvalidResourcePropertyQName e){
			System.out.println(e.toString());
		}
		if(results==null){
			System.out.println("true");
		}
		System.out.println(results.toXML());
		
				System.out.println("ĳԴ");
		Metaset source = new Metaset("insert");
		Metaset insert = new Metaset("someElement");
		source.add(new Attribute(new QName("xmlns"),"http://example.com"));
		insert.setValue("234");
		source.add(insert);
		insert.setFather(source);
		try{
			re = resource.insert(source,id);
			System.out.println(re.toXML());
		}catch(ResourceUnknownFault e){
			System.out.println(e.toString());
		}
		
				System.out.println("ѯղŲԴ");
		QName qnameCheck = new QName("someElement");
		qnameCheck.setNameSpace("http://example.com");
		Metaset checkSet = null;
		try{
			checkSet = resource.getResourceProperty(qnameCheck,id);
		}catch(ResourceUnknownFault e){
			System.out.println(e.toString());
		}catch(InvalidResourcePropertyQName e){
			System.out.println(e.toString());
		}
		System.out.println(checkSet.toXML());
		
				System.out.println("ɾĳ");
		QName de = new QName("http://example.com/diskDrive","BlockSize");
		try{
		re = resource.delete(de,id);
		System.out.println(re.toXML());
		}catch(ResourceUnknownFault e){
			System.out.println(e.toString());
		}catch(UnableToModifyResourceProperty e){
			System.out.println(e.toString());
		}catch(InvalidResourcePropertyQName e){
			System.out.println(e.toString());
		}
		
				System.out.println("޸ĳ");
		Metaset update = new Metaset("NumberOfBlocks");
		update.add(new Attribute(new QName("xmlns"),"http://example.com/diskDrive"));
			update.setValue("256");
		try{
		re = resource.update(update,id);
		System.out.println(re.toXML());
		}catch(ResourceUnknownFault e){
			System.out.println(e.toString());
		}catch(InvalidResourcePropertyQName e){
			System.out.println(e.toString());
		}
		
				System.out.println("޸Ķ");
		Metaset updates = new Metaset("Update");
		updates.add(new Attribute(new QName("xmlns"),"http://example.com/diskDrive"));
		Metaset up1 = new Metaset("NumberOfBlocks");
		up1.setValue("386");
		Metaset up2 = new Metaset("Manufacturer");
		up2.setValue("SUN");
		updates.add(up1);
		up1.setFather(updates);
		updates.add(up2);
		up2.setFather(updates);
		try{
		re = resource.update(updates,id);
		System.out.println(re.toXML());
		}catch(ResourceUnknownFault e){
			System.out.println(e.toString());
		}catch(InvalidResourcePropertyQName e){
			System.out.println(e.toString());
		}
		
				System.out.println("ʱ");
		Calendar time = Calendar.getInstance();
		time.set(2005,8,25,9,55,00);
		try{
		Metaset resp = resource.setTerminationTime(id,time);
		System.out.println(time.getTime().toLocaleString());
		System.out.println(resp.toXML());
		}catch(ResourceUnknownFault e){
			System.out.println(e.toString());
		}
		
				System.out.println("ͨѯԵķʽȡCurrentTimeTerminationTime");
		QName qnameCT = new QName("CurrentTime");
		QName qnameTT = new QName("TerminationTime");
		qnameCT.setNameSpace("http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceLifetime-1.2-draft-01.xsd");
		qnameTT.setNameSpace("http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceLifetime-1.2-draft-01.xsd");
		QName[] nameV = new QName[2];
		nameV[0] = qnameCT;
		nameV[1] = qnameTT;
		try{
			Metaset responseTime = resource.getMultipleResourceProperties(nameV,id);
			System.out.println(responseTime.toXML());
		}catch(ResourceUnknownFault e){
			System.out.println(e.toString());
		}catch(InvalidResourcePropertyQName fault){
			System.out.println(fault.toString());
		}
		
							
				System.out.println("ȡǰʱ");
		Calendar curTime = resource.getCurrentTime(id);
		System.out.println(curTime.getTime().toLocaleString());
		
				System.out.println("Դ");
		try{
			Metaset des = resource.destroy(id);
			System.out.println(des.toXML());
		}
		catch(ResourceUnknownFault e){
			System.out.println(e.toString());
		}catch(ResourceNotDestroyedFault fault){
			System.out.println(fault.toString());
		}
		
		
				System.out.println("ٺٲѯĳԴ");
		try{
			Metaset result1 = resource.getResourceProperty(name,id);
			System.out.println(result.toXML());
		}catch(ResourceUnknownFault e){
			System.out.println(e.toString());
		}catch(InvalidResourcePropertyQName fault){
			System.out.println(fault.toString());
		}
	}
}
